// © 2020 ETH Zurich, Mechanics and Materials Lab
//
// This file is part of ae108.
//
// ae108 is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// ae108 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ae108. If not, see <https://www.gnu.org/licenses/>.

#include "Shape_Test.h"
#include "ae108/elements/shape/Hexa8.h"
#include "ae108/elements/shape/get_points.h"
#include <gmock/gmock.h>
#include <gtest/gtest.h>

using testing::DoubleEq;
using testing::ElementsAre;
using testing::Test;
using testing::Types;

namespace ae108::elements::shape {
namespace {

using Configurations = Types<Hexa8>;

INSTANTIATE_TYPED_TEST_SUITE_P(Hexa8_Test, Shape_Test, Configurations);

struct Hexa8_Test : Test {
  using Shape = Hexa8;
};

TEST_F(Hexa8_Test, points_are_correct) {
  const auto points = get_points<Shape>();

  EXPECT_THAT(
      points,
      ElementsAre(ElementsAre(DoubleEq(-1.), DoubleEq(-1.), DoubleEq(-1.)),
                  ElementsAre(DoubleEq(1.), DoubleEq(-1.), DoubleEq(-1.)),
                  ElementsAre(DoubleEq(1.), DoubleEq(1.), DoubleEq(-1.)),
                  ElementsAre(DoubleEq(-1.), DoubleEq(1.), DoubleEq(-1.)),
                  ElementsAre(DoubleEq(-1.), DoubleEq(-1.), DoubleEq(1.)),
                  ElementsAre(DoubleEq(1.), DoubleEq(-1.), DoubleEq(1.)),
                  ElementsAre(DoubleEq(1.), DoubleEq(1.), DoubleEq(1.)),
                  ElementsAre(DoubleEq(-1.), DoubleEq(1.), DoubleEq(1.))));
}

} // namespace
} // namespace ae108::elements::shape